function [om, om_st, tau2, lam2] = get_om_tau2_lam2( lam20, y, G, bigSig, om, tau2, mu, pi0 )

% This function samples omega's and the hyperparameters conditional on all
% other parameters and data.  If no mu's are provided, they are assumed to
% be zero.  If prior probabilities of exclusion (pi0) are provided,
% sampling will proceed without the hierarhical lasso shrinkage.
%
% NOTICE:
% -------
% Permission is hereby granted to use this code freely for academic
% purposes only, provided that the paper is duly cited as:
%
% Eisenstat, E., Chan, J. C. C. and R. W. Strachan (2015). "Stochastic Model
% Specification Search for Time-Varying Parameter VARs," Econometric
% Reviews, forthcoming.
%
% Any other use of this code, particularly for comercial purposes, is
% strictly prohibited without the prior written consent of the authors.
% Furthermore, this code comes without technical support of any kind.  It
% is expected to reproduce the results reported in the paper.  However,
% beware that the notation may not the match text.  Likewise, it is not
% gauranteed to work under different hardware and / or software settings,
% other than those used for its original design and implementation.
%
% Under no circumstances will the authors be held responsible for any use
% (or misuse) of this code in any way.  If you do not agree to the above,
% you do not have permission to use this code.

m = length( om );
om_st = zeros( m, 1 );
lasso = true;

if nargin < 7
    % for lasso specification
    % mu_j = 0 and pi0_j = .5
    mu = zeros( m, 1 );
    pi0 = repmat( 0.5, m, 1 );
elseif nargin < 8
    % if mu is given with lasso prior
    % set pi0_j = Phi( -mu_j / tau_j )
    pi0 = normcdf( -mu ./ sqrt( tau2 ) );
else
    % if pi0 are provided, don't use lasso prior
    % note: this is reserved for testing only
    lasso = false;
end

if lasso
    % sample lam2 first
    lam2 = gamrnd( lam20( 1 ) + m, 1 / ( lam20( 2 ) + sum( tau2 ) / 2 ) );
end

for j = 1:m
    nj = [1:( j - 1 ) ( j + 1 ):m];
    Gnj = G( :, nj );
    Gj = G( :, j );
    vj = y - Gnj * om( nj );
    
    % compute posterior quantities
    tau2j_hat = 1 / ( 1 / tau2( j ) + Gj' * bigSig * Gj );
    muj_hat = tau2j_hat * ( mu( j ) / tau2( j ) + Gj' * bigSig * vj );

    % compute posterior probability of w_j = 0
    ln_xi0 = log( 1 - pi0( j ) ) - log( pi0( j ) );
    ln_xi1 = log( normcdf( muj_hat / sqrt( tau2j_hat ) ) ) + log( tau2j_hat ) / 2;
    ln_xi2 =  log( normcdf( mu( j ) / sqrt( tau2( j ) ) ) ) + log( tau2( j ) ) / 2;
    ln_xi3 = ( muj_hat ^ 2 / tau2j_hat - mu( j ) ^ 2 / tau2( j ) ) / 2;
    pij = 1 / ( 1 + exp( ln_xi0 + ln_xi1 - ln_xi2 + ln_xi3 ) );

    om_st( j ) = tnr( muj_hat, sqrt( tau2j_hat ), 0, Inf );
    if pij > rand
        om_st( j ) = tnr( mu( j ), sqrt( tau2( j ) ), -Inf, 0 );
    end
    om( j ) = om_st( j ) * ( om_st( j ) > 0 );
    
    if lasso
        tau2( j ) = 1 / igrand( sqrt( lam2 ) / abs( om_st( j ) - mu( j ) ), lam2 );
    end
end
