function [ab2, c, structvars, IR, err] = ir_BP2002_vecm_sv( beta, sigma, cidx, t_start, s, a1, p, D, cum_ir, tax_first )

% This function computes impulses responses for the VECM models with
% stochastic volatility. Structural parameters are recovered from the
% estimated parameters using the procedure described in the appendix.
%
% NOTICE:
% -------
% Permission is hereby granted to use this code freely for academic
% purposes only, provided that the paper is duly cited as:
%
% Eisenstat, E., Chan, J. C. C. and R. W. Strachan (2015). "Stochastic Model
% Specification Search for Time-Varying Parameter VARs," Econometric
% Reviews, forthcoming.
%
% Any other use of this code, particularly for comercial purposes, is
% strictly prohibited without the prior written consent of the authors.
% Furthermore, this code comes without technical support of any kind.  It
% is expected to reproduce the results reported in the paper.  However,
% beware that the notation may not the match text.  Likewise, it is not
% gauranteed to work under different hardware and / or software settings,
% other than those used for its original design and implementation.
%
% Under no circumstances will the authors be held responsible for any use
% (or misuse) of this code in any way.  If you do not agree to the above,
% you do not have permission to use this code.

if nargin == 8
    % assume non-cumulative IR and taxes first by default
    cum_ir = false;
    tax_first = true;
end

if nargin == 9
    % assume taxes first by default
    tax_first = true;
end

% allocate space
[n, ~, nsims] = size( sigma );
k = ( size( beta, 1 ) - 0.5 * n * ( n - 1 ) ) / n;
bidx = logical( 1 - cidx );

% for constructing the contemporaneous coefficients
B0t = eye( n ); % this is actually B0_t'
ltidx = logical( triu( ones( n, n ) ) - eye( n ) );

ab2 = zeros( 1, nsims );
c = zeros( 2, nsims );
structvars = zeros( 3, nsims );
err = zeros( 1, nsims );
IR = zeros( n, n, 1 + s, nsims );

disp( 'Simulating impulse responses...' );
txtlen = 0; tic;
for i = 1:nsims
    % decompose the reduced form cov matrix
    % using identification scheme in BP2002
    psi = -[a1( 1 ) + ( a1( length( 1 ) ) - a1( 1 ) ) * rand; 0];
    B0t( ltidx ) = beta( cidx, t_start, i );
    Qt = B0t * diag( 1 ./ sigma( :, t_start, i ) ) * B0t';
    [Kap, negc, Delta, Xi, s2_3, err( i )] = solve_struct( Qt, psi, tax_first );
    
    % save draws of structural params + variances
    ab2( i ) = Delta( 1, 2 ) + Delta( 2, 1 ); % note: either Delta12 or Delta21 is 0
    c( :, i ) = - negc;
    structvars( :, i ) = [diag( Xi ); s2_3];
    
    % compute impulse responses for all combinations
    m = size( D, 1 ); % the cointegration dim
    iKap = inv( Kap ); % note: this is just 3x3
    normKap = iKap * diag( 1 ./ diag( iKap ) ); % normalize shocks
    F = eye( n * ( p + 1 ) ); % note: VECM induces an extra lag in the VAR
    IR( :, :, 1, i ) = normKap; % simultaneous response
    for dt = 1:s
        B0t( ltidx ) = beta( cidx, t_start + dt, i );
        Bt = ( reshape( beta( logical( bidx ), t_start + dt, i ), k, n ) / B0t )';
        Ct = [Bt( :, k - p * n + 1:end ) zeros( n, n )];
        Ct = Ct - [-( eye( n ) + Bt( :, k - p * n - m + 1:k - p * n ) * D ) Bt( :, k - p * n + 1:end )];
        F = [Ct; [eye( n * p ) zeros( n * p, n )]] * F;
        IR( :, :, 1 + dt, i ) = cum_ir * IR( :, :, dt, i ) + F( 1:n, 1:n ) * normKap;
    end
    
    % show progress
    txtlen = showprog( i, nsims, 100, txtlen );
end

fprintf( '\nIR simulation completed after %5.3f minutes.\n', toc / 60 );
